@@ -26,9 +26,12 @@
 exports.standard = {
     wrapOverride: function( method, super_method, cid, getInst )
     {
-        return function()
+        var retf = function()
         {
-            var context = getInst( this, cid ) || this,
+            // we need some sort of context in order to set __super; it may
+            // be undefined per strict mode requirements depending on how
+            // the method was invoked
+            var context = getInst( this, cid ) || this || {},
                 retval  = undefined
             ;
 
@@ -55,6 +58,9 @@ exports.standard = {
 
             return retval;
         };
+
+        retf.super = super_method;
+        return retf;
     },
 
 
