@@ -23,6 +23,10 @@ var AbstractClass = require( './class_abstract' ),
     ClassBuilder  = require( './ClassBuilder' ),
     Interface     = require( './interface' );
 
+
+function _fvoid() {};
+
+
 /**
  * Trait constructor / base object
  *
@@ -96,11 +100,13 @@ function _createStaging( name )
 Trait.extend = function( dfn )
 {
     // we may have been passed some additional metadata
-    var meta = this.__$$meta || {};
+    var meta = ( this || {} ).__$$meta || {};
 
     // store any provided name, since we'll be clobbering it (the definition
     // object will be used to define the hidden abstract class)
-    var name = dfn.__name || '(Trait)';
+    var name    = dfn.__name || '(Trait)',
+        type    = _getTraitType( dfn ),
+        isparam = ( type === 'param' );
 
     // augment the parser to handle our own oddities
     dfn.___$$parser$$ = {
@@ -116,10 +122,41 @@ Trait.extend = function( dfn )
     // give the abstract trait class a distinctive name for debugging
     dfn.__name = '#AbstractTrait#';
 
-    function TraitType()
-    {
-        throw Error( "Cannot instantiate trait" );
-    };
+    // if __mixin was provided,in the definition, then we should crate a
+    // paramaterized trait
+    var Trait = ( isparam )
+        ? function ParameterTraitType()
+        {
+            // duplicate ars in a way that v8 can optimize
+            var args = [], i = arguments.length;
+            while ( i-- ) args[ i ] = arguments[ i ];
+
+            var AT = function ArgumentTrait()
+            {
+                throw Error( "Cannot re-configure argument trait" );
+            };
+
+            // TODO: mess!
+            AT.___$$mixinargs = args;
+            AT.__trait        = 'arg';
+            AT.__acls         = Trait.__acls;
+            AT.__ccls         = Trait.__ccls;
+            AT.toString       = Trait.toString;
+            AT.__mixinImpl    = Trait.__mixinImpl;
+            AT.__isInstanceOf = Trait.__isInstanceOf;
+
+            // mix in the argument trait instead of the original
+            AT.__mixin = function( dfn, tc, base )
+            {
+                mixin( AT, dfn, tc, base );
+            };
+
+            return AT;
+        }
+        : function TraitType()
+        {
+            throw Error( "Cannot instantiate non-parameterized trait" );
+        };
 
     // implement interfaces if indicated
     var base = AbstractClass;
@@ -131,33 +168,50 @@ Trait.extend = function( dfn )
     // and here we can see that traits are quite literally abstract classes
     var tclass = base.extend( dfn );
 
-    TraitType.__trait  = true;
-    TraitType.__acls   = tclass;
-    TraitType.__ccls   = null;
-    TraitType.toString = function()
+    Trait.__trait  = type;
+    Trait.__acls   = tclass;
+    Trait.__ccls   = null;
+    Trait.toString = function()
     {
         return ''+name;
     };
 
     // invoked to trigger mixin
-    TraitType.__mixin = function( dfn, tc, base )
+    Trait.__mixin = function( dfn, tc, base )
     {
-        mixin( TraitType, dfn, tc, base );
+        mixin( Trait, dfn, tc, base );
     };
 
     // mixes in implemented types
-    TraitType.__mixinImpl = function( dest_meta )
+    Trait.__mixinImpl = function( dest_meta )
     {
         mixinImpl( tclass, dest_meta );
     };
 
     // TODO: this and the above should use util.defineSecureProp
-    TraitType.__isInstanceOf = Interface.isInstanceOf;
+    Trait.__isInstanceOf = Interface.isInstanceOf;
 
-    return TraitType;
+    return Trait;
 };
 
 
+/**
+ * Retrieve a string representation of the trait type
+ *
+ * A trait is parameterized if it has a __mixin method; otherwise, it is
+ * standard.
+ *
+ * @param   {Object}  dfn  trait definition object
+ * @return  {string}  trait type
+ */
+function _getTraitType( dfn )
+{
+    return ( typeof dfn.__mixin === 'function' )
+        ? 'param'
+        : 'std';
+}
+
+
 /**
  * Verifies trait member restrictions
  *
@@ -295,8 +349,7 @@ function createImplement( ifaces, name )
 /**
  * Determines if the provided value references a trait
  *
- * @param  {*}  trait  value to check
- *
+ * @param   {*}        trait  value to check
  * @return  {boolean}  whether the provided value references a trait
  */
 Trait.isTrait = function( trait )
@@ -305,6 +358,32 @@ Trait.isTrait = function( trait )
 };
 
 
+/**
+ * Determines if the provided value references a parameterized trait
+ *
+ * @param   {*}        trait  value to check
+ * @return  {boolean}  whether the provided value references a param trait
+ */
+Trait.isParameterTrait = function( trait )
+{
+    return !!( ( trait || {} ).__trait === 'param' );
+};
+
+
+/**
+ * Determines if the provided value references an argument trait
+ *
+ * An argument trait is a configured parameter trait.
+ *
+ * @param   {*}        trait  value to check
+ * @return  {boolean}  whether the provided value references an arg trait
+ */
+Trait.isArgumentTrait = function( trait )
+{
+    return !!( ( trait || {} ).__trait === 'arg' );
+};
+
+
 /**
  * Create a concrete class from the abstract trait class
  *
@@ -398,11 +477,14 @@ function createVirtProxy( acls, dfn )
             ? 'public'
             : 'protected';
 
+        var srcmethod = acls.___$$methods$$[ vis ][ f ],
+            plen      = srcmethod.__length;
+
         // this is the aforementioned proxy method; see the docblock for
         // more information
-        dfn[ vis + ' virtual override ' + f ] = ( function()
+        dfn[ vis + ' virtual override ' + f ] = ( function( f )
         {
-            return function()
+            var retf = function()
             {
                 var pmo = this.___$$pmo$$,
                     o   = pmo[ f ];
@@ -413,17 +495,23 @@ function createVirtProxy( acls, dfn )
                     ? o.apply( pmo, arguments )
                     : this.__super.apply( this, arguments );
             };
+
+            retf.__length = plen;
+            return retf;
         } )( f );
 
         // this guy bypasses the above virtual override check, which is
         // necessary in certain cases to prevent infinte recursion
-        dfn[ vis + ' virtual __$$' + f ] = ( function( f )
+        dfn[ vis + ' virtual __$$' + f ] = ( function( method )
         {
-            return function()
+            var retf = function()
             {
-                return this.___$$parent$$[ f ].apply( this, arguments );
+                return method.apply( this, arguments );
             };
-        } )( f );
+
+            retf.__length = plen;
+            return retf;
+        } )( srcmethod );
     }
 }
 
@@ -451,6 +539,24 @@ function mixin( trait, dfn, tc, base )
     // retrieve the private member name that will contain this trait object
     var iname = addTraitInst( trait, dfn, tc, base );
 
+    // TODO: this should not be necessary for dfn metadata
+    dfn[ 'weak virtual ___$$ctor$pre$$' ]  = _fvoid;
+    dfn[ 'weak virtual ___$$ctor$post$$' ] = _fvoid;
+
+    // TODO: this is a kluge; generalize and move
+    // this ensures __construct is called before __mixin when mixing into
+    // the base class
+    if ( base === ClassBuilder.ClassBase )
+    {
+        dfn[ 'virtual override ___$$ctor$post$$' ] = _tctorApply;
+        dfn[ 'virtual override ___$$ctor$pre$$' ]  = _fvoid;
+    }
+    else
+    {
+        dfn[ 'virtual override ___$$ctor$post$$' ]  = _fvoid;
+        dfn[ 'virtual override ___$$ctor$pre$$' ] = _tctorApply;
+    }
+
     // recursively mix in trait's underlying abstract class (ensuring that
     // anything that the trait inherits from is also properly mixed in)
     mixinCls( acls, dfn, iname );
@@ -533,6 +639,14 @@ function mixMethods( src, dest, vis, iname )
             continue;
         }
 
+        // TODO: generalize
+        // __mixin is exclusive to the trait (private-ish, but can be
+        // invoked publically internally)
+        if ( f === '__mixin' )
+        {
+            continue;
+        }
+
         // TODO: this is a kluge; we'll use proper reflection eventually,
         // but for now, this is how we determine if this is an actual method
         // vs. something that just happens to be on the visibility object
@@ -581,7 +695,7 @@ function mixMethods( src, dest, vis, iname )
             // beacuse we are not proxying to a method of the same name)
             dest[ pname ] = ( function( f )
             {
-                return function()
+                var retf = function()
                 {
                     var pdest = this[ iname ];
 
@@ -596,6 +710,9 @@ function mixMethods( src, dest, vis, iname )
                         ? this
                         : ret;
                 };
+
+                retf.__length = src[ f ].__length;
+                return retf;
             } )( f );
         }
     }
@@ -658,15 +775,24 @@ function addTraitInst( T, dfn, tc, base )
  * resulting objects assigned to their rsepective pre-determined field
  * names.
  *
+ * The MIXINARGS are only useful in the case of parameterized trait.
+ *
  * This will lazily create the concrete trait class if it does not already
  * exist, which saves work if the trait is never used.
  *
- * @param  {Object}  tc    trait class list
- * @param  {Class}   base  target supertype
+ * Note that the private symbol used to encapsulate class data must be
+ * passed to this function to provide us access to implementation details
+ * that we really shouldn't be messing around with. :) In particular, we
+ * need access to the protected visibility object, and there is [currently]
+ * no API for doing so.
+ *
+ * @param  {Object}  tc       trait class list
+ * @param  {Class}   base     target supertype
+ * @param  {Symbol}  privsym  symbol used as key for encapsulated data
  *
  * @return  {undefined}
  */
-function tctor( tc, base )
+function tctor( tc, base, privsym )
 {
     // instantiate all traits and assign the object to their
     // respective fields
@@ -681,11 +807,17 @@ function tctor( tc, base )
         // (but not private); in return, we will use its own protected
         // visibility object to gain access to its protected members...quite
         // the intimate relationship
-        this[ f ] = C( base, this.___$$vis$$ ).___$$vis$$;
+        this[ f ] = C( base, this[ privsym ].vis )[ privsym ].vis;
+
+        // this has been previously validated to ensure that it is a
+        // function
+        this[ f ].__mixin && this[ f ].__mixin.apply(
+            this[ f ], T.___$$mixinargs
+        );
     }
 
     // if we are a subtype, be sure to initialize our parent's traits
-    this.__super && this.__super();
+    this.__super && this.__super( privsym );
 };
 
 
@@ -702,12 +834,18 @@ function tctor( tc, base )
  */
 function createTctor( tc, base )
 {
-    return function()
+    return function( privsym )
     {
-        return tctor.call( this, tc, base );
+        return tctor.call( this, tc, base, privsym );
     };
 }
 
 
+function _tctorApply()
+{
+    this.___$$tctor$$.apply( this, arguments );
+}
+
+
 module.exports = Trait;
 
