@@ -19,16 +19,31 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+/**
+ * Console to use for logging
+ *
+ * This reference allows an alternative console to be used. Must contain
+ * warn() or log() methods.
+ *
+ * TODO: This needs to be moved into a facade, once more refactoring can be
+ * done; it was moved out of warn during its refactoring.
+ *
+ * @type {Object}
+ */
+var _console = ( typeof console !== 'undefined' ) ? console : undefined;
+
 var util         = require( './util' ),
     ClassBuilder = require( './ClassBuilder' ),
 
-    warn    = require( './warn' ),
-    Warning = warn.Warning,
+    warn        = require( './warn' ),
+    Warning     = warn.Warning,
+    log_handler = warn.LogHandler( _console ),
 
     MethodWrapperFactory = require( './MethodWrapperFactory' ),
     wrappers             = require( './MethodWrappers' ).standard,
 
     class_builder = ClassBuilder(
+        log_handler,
         require( './MemberBuilder' )(
             MethodWrapperFactory( wrappers.wrapNew ),
             MethodWrapperFactory( wrappers.wrapOverride ),
@@ -36,7 +51,7 @@ var util         = require( './util' ),
             require( './MemberBuilderValidator' )(
                 function( warning )
                 {
-                    warn.handle( Warning( warning ) );
+                    log_handler.handle( Warning( warning ) );
                 }
             )
         ),
@@ -148,18 +163,33 @@ module.exports.use = function( traits )
 };
 
 
+var _dummyclass = { prototype: {} };
+var _dummyinst  = { constructor: { prototype: {} } };
+
 /**
  * Determines whether the provided object is a class created through ease.js
  *
+ * TODO: delegate to ClassBuilder
+ *
  * @param  {Object}  obj  object to test
  *
  * @return  {boolean}  true if class (created through ease.js), otherwise false
  */
 module.exports.isClass = function( obj )
 {
-    obj = obj || {};
+    obj = obj || _dummyclass;
 
-    return ( obj.prototype instanceof ClassBuilder.ClassBase )
+    if ( !obj.prototype )
+    {
+        return false;
+    }
+
+    var meta = ClassBuilder.getMeta( obj );
+
+    // TODO: we're checking a random field on the meta object; do something
+    // proper
+    return ( ( ( meta !== null ) && meta.implemented )
+        || ( obj.prototype instanceof ClassBuilder.ClassBase ) )
         ? true
         : false
     ;
@@ -170,6 +200,8 @@ module.exports.isClass = function( obj )
  * Determines whether the provided object is an instance of a class created
  * through ease.js
  *
+ * TODO: delegate to ClassBuilder
+ *
  * @param  {Object}  obj  object to test
  *
  * @return  {boolean}  true if instance of class (created through ease.js),
@@ -177,11 +209,10 @@ module.exports.isClass = function( obj )
  */
 module.exports.isClassInstance = function( obj )
 {
-    obj = obj || {};
+    obj = obj || _dummyinst;
 
-    return ( obj instanceof ClassBuilder.ClassBase )
-        ? true
-        : false;
+    // if the constructor is a class, then we must be an instance!
+    return module.exports.isClass( obj.constructor );
 };
 
 
