@@ -492,15 +492,19 @@ function attachInstanceOf( iface )
  */
 function _isInstanceOf( type, instance )
 {
+    // we are interested in the class's metadata, not the instance's
+    var proto = instance.constructor;
+
     // if no metadata are available, then our remaining checks cannot be
     // performed
-    if ( !instance.__cid || !( meta = ClassBuilder.getMeta( instance ) ) )
+    var meta;
+    if ( !instance.__cid || !( meta = ClassBuilder.getMeta( proto ) ) )
     {
         return false;
     }
 
-    implemented = meta.implemented;
-    i           = implemented.length;
+    var implemented = meta.implemented,
+        i           = implemented.length;
 
     // check implemented interfaces et. al. (other systems may make use of
     // this meta-attribute to provide references to types)
